/***************************************************************************
                          kvoulumerendererdoc.cpp  -  description
                             -------------------
    begin                : Mon Jan 21 11:14:26 CET 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kio/job.h>
#include <kio/netaccess.h>

// application specific includes
#include "kvoulumerenderer.h"
#include "kvoulumerendererview.h"
#include "kvoulumerendererdoc.h"
#include "voulumeimporter.h"
#include "voulumeimporterfactory.h"

KvoulumeRendererDoc::KvoulumeRendererDoc(KvoulumeRendererApp *appobj)
{
  this->appobj = appobj;
  pViewList = new QList<KvoulumeRendererView>;
  pViewList->setAutoDelete(false);
}

KvoulumeRendererDoc::~KvoulumeRendererDoc()
{
  delete pViewList;
  if(voulumedata) delete voulumedata;
}

void KvoulumeRendererDoc::addView(KvoulumeRendererView *view)
{
  pViewList->append(view);
  connect(this,SIGNAL(docSizeChanged()),view,SLOT(slotDocSizeChanged()));
	changedViewList();
}

void KvoulumeRendererDoc::removeView(KvoulumeRendererView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void KvoulumeRendererDoc::changedViewList(){	
	
	KvoulumeRendererView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->slotSetBaseCaption(URL().fileName());
	}
	else{	
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->slotSetBaseCaption(QString(URL().fileName()+":%1").arg(i));	
	}
}

bool KvoulumeRendererDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void KvoulumeRendererDoc::updateAllViews(KvoulumeRendererView *sender)
{
  KvoulumeRendererView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void KvoulumeRendererDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& KvoulumeRendererDoc::URL() const
{
  return doc_url;
}

void KvoulumeRendererDoc::closeDocument()
{
  KvoulumeRendererView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
}

bool KvoulumeRendererDoc::newDocument()
{
  voulumedata = 0;
  width=height=depth=planeoffset=0;
  modified=false;
  datasetdescription = i18n("No dataset has been loaded!");
  return true;
}

bool KvoulumeRendererDoc::openDocument(const KURL &url, const char *format /*=0*/)
{
    VoulumeImporter *importer = appobj->importerfactory.produce(QFileInfo(url.filename()).extension(),this);
    if(!importer)
    {
    	KMessageBox::error(0,i18n("Could not create importer! (out of mem?)"),"KvoulumeRenderer");
    	return false;
    }
    QString log;
	 bool success = importer->import(url,log);
  	 delete importer;
	 if(success)
	 {
		// create text description for current data set
		datasetdescription = i18n("current data set dimensions (width x height x depth): ") + QString::number(width) + QString(" x ") \
		+ QString::number(height) + QString(" x ") + QString::number(depth) + QString("\n");
		datasetdescription += i18n("loaded from ") + url.url();
	   KMessageBox::information(0,log,"KvoulumeRenderer");
  		doc_url=url;
  		modified=false;
  		updateAllViews(0);
  	  }
  	  else KMessageBox::error(0,log,"KvoulumeRenderer");
  	return success;
}

bool KvoulumeRendererDoc::saveDocument(const KURL &url, const char *format /*=0*/)
{
//	QFile f( filename );
//	if ( !f.open( IO_WriteOnly ) )
//		return false;
//
//  /////////////////////////////////////////////////
//  // TODO: Add your document saving code here
//  /////////////////////////////////////////////////
//
//  f.close();
//
//  modified=false;
//	m_filename=filename;
//	m_title=QFileInfo(f).fileName();
  return true;
}

void KvoulumeRendererDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KvoulumeRendererDoc::canCloseFrame(KvoulumeRendererView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		KURL saveURL;
  	switch(KMessageBox::warningYesNoCancel(pFrame, i18n("The current file has been modified.\n"
                          "Do you want to save it?"),URL().fileName()))
    {
			case KMessageBox::Yes:
				if(URL().fileName().contains(i18n("Untitled")))
				{
					saveURL=KFileDialog::getSaveURL(QDir::currentDirPath(),
                      i18n("*|All files"), pFrame, i18n("Save as..."));
          if(saveURL.isEmpty())
          	return false;
				}
				else
						saveURL=URL();
					
				if(!saveDocument(saveURL))
				{
 					switch(KMessageBox::warningYesNo(pFrame,i18n("Could not save the current document !\n"
																												"Close anyway ?"), i18n("I/O Error !")))
 					{
 						case KMessageBox::Yes:
 							ret=true;
 						case KMessageBox::No:
 							ret=false;
 					}	        			
				}
				else
					ret=true;
				break;
			case KMessageBox::No:
				ret=true;
				break;
			case KMessageBox::Cancel:
			default:
				ret=false; 				
				break;
		}
	}
	else
		ret=true;
		
	return ret;
}

/**
	* tries to resize the content space to a width x height x depth cube; returns
	* true on success
	*/

bool KvoulumeRendererDoc::resize(unsigned int width,unsigned int height,unsigned int depth)
{
	// it would be deadly if we kill the voulume data set when it is being rendered!
	datamutex.lock();
	// delete old dataset if it exists
	if(voulumedata) delete voulumedata;
	this->width=this->height=this->depth=0;
	if((voulumedata = new unsigned char[width*height*depth]) != 0)
	{
		this->width = width;
		this->height = height;
		this->depth = depth;
		planeoffset = width*height;
		emit docSizeChanged();
	}
	datamutex.unlock();
	return (voulumedata != 0);
}
