/***************************************************************************
                          voulumeimporterfactory.h  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Ruediger Knoerig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VOULUMEIMPORTERFACTORY_H
#define VOULUMEIMPORTERFACTORY_H

#include <map>
#include <qstring.h>
#include <qstringlist.h>

/**factory class for the voulume importer classes
  *@author Ruediger Knoerig
  */

class VoulumeImporter;
class KvoulumeRendererDoc;

class VoulumeImporterFactory {
public: 
	VoulumeImporterFactory();
	~VoulumeImporterFactory();
  	/** tries to produce a voulume importer from the given id
  		* @param id  (e.g. index of list returned by getSupportedFormatList()
  		* @param docobj pointer to the doc object the importer should work on
  		* @return hopefully a pointer to a usable importer...
  		*/
  	VoulumeImporter *produce(unsigned int id,KvoulumeRendererDoc *docobj);
  	/** tries to produce a voulume importer from a given filename type extension
  	  	* @param typeextension filename type extension for the desired format
  	   	* @param docobj pointer to the doc object the importer should work on
  	 	* @return hopefully a pointer to a usable importer...
  		*/
  	VoulumeImporter *produce(QString typeextension,KvoulumeRendererDoc *docobj);
  	/** method returning a list with the full names of all supported formats */  	
  	QStringList getSupportedFormatList() { return description; }
   /** method returning a QString suitable as filter argument for KDE's file selection dialogs */
   QString getFilterString() { return filterstring; }
protected:
   	/** map storing the association between file extensions and file type ID's */
    map<QString,unsigned int> mimetypes;
    /** string list containing a long description for each format ID */
   	QStringList description;
    /** QString suitable as filter argument for KDE's file selection dialogs */
    QString filterstring;
};

#endif
